@@ -1,9 +1,8 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
-# Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
-# Copyright (C) 2002-2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2005 Jrg Lehmann <joergl@users.sourceforge.net>
+# Copyright (C) 2002-2005 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -19,25 +18,23 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-""" PyX := PostScript + Python + TeX
-
-PyX is a python package for the creation of encapsulated PostScript
-figures. It provides both an abstraction of PostScript and a TeX/LaTeX
-interface. Complex tasks like 2d plots in publication-ready quality
-are build out of these primitives.
+"""Python graphics package
 
+PyX is a Python package for the creation of PostScript and PDF files. It
+combines an abstraction of the PostScript drawing model with a TeX/LaTeX
+interface. Complex tasks like 2d and 3d plots in publication-ready quality are
+built out of these primitives.
 """
 
 import version
 __version__ = version.version
 
-__all__ = ["attr", "box", "bitmap", "canvas", "color", "connector", "deco", "deformer", "epsfile", "graph", "path",
-           "style", "trafo", "text", "unit"]
+__all__ = ["attr", "box", "bitmap", "canvas", "color", "connector", "deco", "deformer", "document",
+           "epsfile", "graph", "path", "pattern", "style", "trafo", "text", "unit"]
 
 
 # automatically import main modules into pyx namespace
 for module in __all__:
     __import__(module, globals(), locals(), [])
-
