@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
@@ -20,29 +19,23 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 # check for an isinstance which accepts both a class and a sequence of classes
 # as second argument and emulate this behaviour if necessary
 try:
     isinstance(1, (int, float))
 except TypeError:
-    # workaround for Python 2.1 and older
+    # workaround for Python 2.1
     _isinstance = isinstance
     def isinstance(instance, clsarg):
-        # we explicitely check for clsarg being a sequence, because
-        # a Leap Before Look pattern, i.e., just trying out
-        # _isinstance(instance, clsarg) within an try/except
-        # block, may mask a TypeError occuring when clsarg is
-        # not a class
-        import helper
-        if not helper.issequence(clsarg):
+        import types
+        if _isinstance(clsarg, types.ClassType):
             return _isinstance(instance, clsarg)
-        else:
-            for cls in clsarg:
-                if _isinstance(instance, cls):
-                    return 1
-            return 0
+        for cls in clsarg:
+            if _isinstance(instance, cls):
+                return 1
+        return 0
 
 #
 # some helper functions for the attribute handling
@@ -171,8 +164,6 @@ class clearclass(attr):
         return [attr for attr in attrs if not isinstance(attr, self.clearclass)]
 
 
-# XXX is _clear a good choice?
-
 class _clear(attr):
 
     """a special attribute which removes all predecessing attributes
@@ -261,4 +252,6 @@ class changelist(changeattr):
             return self.attrs[index % len(self.attrs)]
         elif index < len(self.attrs):
             return self.attrs[index]
+        else:
+            return None
 
