@@ -1,9 +1,8 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
 # Copyright (C) 2003-2004 Jrg Lehmann <joergl@users.sourceforge.net>
-# Copyright (C) 2003-2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2003-2005 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -19,12 +18,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 import ConfigParser, os.path
 import siteconfig
 
-cflist = [siteconfig.pyxrc,  os.path.expanduser("~/.pyxrc")]
+cflist = [os.path.join(siteconfig.pyxrcdir, "pyxrc"),  os.path.expanduser("~/.pyxrc")]
 
 config = ConfigParser.ConfigParser()
 config.read(cflist)
