@@ -1,8 +1,7 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
-# Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
+# Copyright (C) 2003-2006 Michael Schindler <m-schindler@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -18,12 +17,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
 import math
 from math import pi, sin, cos, atan2, tan, hypot, acos, sqrt
-import path, unit, helper
+import path, unit, mathutils, normpath
 try:
     from math import radians, degrees
 except ImportError:
@@ -36,7 +35,7 @@ except ImportError:
 ##   helpers
 #########################
 
-class connector_pt(path.normpath):
+class connector_pt(normpath.normpath):
 
     def omitends(self, box1, box2):
         """intersects a path with the boxes' paths"""
@@ -57,26 +56,27 @@ class connector_pt(path.normpath):
 
         # XXX later, this should be done by extended boxes instead of intersecting with circles
         # cut off the start of self
-        center = self.begin_pt()
+        center = self.atbegin_pt()
         cutpath = path.circle_pt(center[0], center[1], dists[0])
         try:
             cutpath = cutpath.normpath()
-        except path.PathException:
+        except normpath.NormpathException:
             pass
         else:
             sp = self.intersect(cutpath)[0]
             self.normsubpaths = self.split(sp[-1:])[1].normsubpaths
 
         # cut off the end of self
-        center = self.end_pt()
+        center = self.atend_pt()
         cutpath = path.circle_pt(center[0], center[1], dists[1])
         try:
             cutpath = cutpath.normpath()
-        except path.PathException:
+        except normpath.NormpathException:
             pass
         else:
             sp = self.intersect(cutpath)[0]
-            self.normsubpaths = self.split(sp[:1])[0].normsubpaths
+            if sp:
+                self.normsubpaths = self.split(sp[:1])[0].normsubpaths
 
 
 ################
@@ -92,7 +92,8 @@ class line_pt(connector_pt):
         self.box2 = box2
 
         connector_pt.__init__(self,
-            [path.normsubpath([path.normline(*(self.box1.center+self.box2.center))], 0)])
+            [path.normsubpath([path.normline_pt(self.box1.center[0], self.box1.center[1],
+                                                self.box2.center[0], self.box2.center[1])], closed=0)])
 
         self.omitends(box1, box2)
         self.shortenpath(boxdists)
@@ -114,49 +115,42 @@ class arc_pt(connector_pt):
         self.box1 = box1
         self.box2 = box2
 
-        rel = (self.box2.center[0] - self.box1.center[0],
-               self.box2.center[1] - self.box1.center[1])
-        distance = hypot(*rel)
+        tangent = (self.box2.center[0] - self.box1.center[0],
+                   self.box2.center[1] - self.box1.center[1])
+        distance = hypot(*tangent)
+        tangent = tangent[0] / distance, tangent[1] / distance
 
-        # usage of bulge overrides the relangle parameter
         if relbulge is not None or absbulge is not None:
-            relangle = None
+            # usage of bulge overrides the relangle parameter
             bulge = 0
-            try: bulge += absbulge
-            except: pass
-            try: bulge += relbulge*distance
-            except: pass
-
-            try: radius = abs(0.5 * (bulge + 0.25 * distance**2 / bulge))
-            except: radius = 10 * distance # default value for too straight arcs
-            radius = min(radius, 10 * distance)
-            center = 2.0*(radius-abs(bulge))/distance
-            center *= 2*(bulge>0.0)-1
-        # otherwise use relangle
+            if absbulge is not None:
+                bulge += absbulge
+            if relbulge is not None:
+                bulge += relbulge*distance
         else:
-            bulge=None
-            try: radius = 0.5 * distance / abs(cos(0.5*math.pi - radians(relangle)))
-            except: radius = 10 * distance
-            try: center = tan(0.5*math.pi - radians(relangle))
-            except: center = 0
+            # otherwise use relangle, which should be present
+            bulge = 0.5 * distance * math.tan(0.5*radians(relangle))
 
-        # up to here center is only the distance from the middle of the
-        # straight connection
-        center = (0.5 * (self.box1.center[0] + self.box2.center[0] - rel[1]*center),
-                  0.5 * (self.box1.center[1] + self.box2.center[1] + rel[0]*center))
-        angle1 = atan2(self.box1.center[1] - center[1], self.box1.center[0] - center[0])
-        angle2 = atan2(self.box2.center[1] - center[1], self.box2.center[0] - center[0])
-
-        # draw the arc in positive direction by default
-        # negative direction if relangle<0 or bulge<0
-        if (relangle is not None and relangle < 0) or (bulge is not None and bulge < 0):
-            connectorpath = path.path(path.moveto_pt(*self.box1.center),
-                                      path.arcn_pt(center[0], center[1], radius, degrees(angle1), degrees(angle2)))
-            connector_pt.__init__(self, connectorpath.normpath().normsubpaths)
+        if abs(bulge) < normpath._epsilon:
+            # fallback solution for too straight arcs
+            connector_pt.__init__(self,
+                [path.normsubpath([path.normline_pt(*(self.box1.center+self.box2.center))], closed=0)])
         else:
-            connectorpath = path.path(path.moveto_pt(*self.box1.center),
-                                      path.arc_pt(center[0], center[1], radius, degrees(angle1), degrees(angle2)))
-            connector_pt.__init__(self, connectorpath.normpath().normsubpaths)
+            radius = abs(0.5 * (bulge + 0.25 * distance**2 / bulge))
+            centerdist = mathutils.sign(bulge) * (radius - abs(bulge))
+            center = (0.5 * (self.box1.center[0] + self.box2.center[0]) + tangent[1]*centerdist,
+                      0.5 * (self.box1.center[1] + self.box2.center[1]) - tangent[0]*centerdist)
+            angle1 = atan2(self.box1.center[1] - center[1], self.box1.center[0] - center[0])
+            angle2 = atan2(self.box2.center[1] - center[1], self.box2.center[0] - center[0])
+
+            if bulge > 0:
+                connectorpath = path.path(path.moveto_pt(*self.box1.center),
+                                          path.arcn_pt(center[0], center[1], radius, degrees(angle1), degrees(angle2)))
+                connector_pt.__init__(self, connectorpath.normpath().normsubpaths)
+            else:
+                connectorpath = path.path(path.moveto_pt(*self.box1.center),
+                                          path.arc_pt(center[0], center[1], radius, degrees(angle1), degrees(angle2)))
+                connector_pt.__init__(self, connectorpath.normpath().normsubpaths)
 
         self.omitends(box1, box2)
         self.shortenpath(boxdists)
@@ -197,7 +191,7 @@ class curve_pt(connector_pt):
         if absangle1 is not None:
             angle1 = radians(absangle1)
         else:
-            angle1 = dangle - radians(relangle1)
+            angle1 = dangle + radians(relangle1)
         if absangle2 is not None:
             angle2 = radians(absangle2)
         else:
@@ -210,7 +204,7 @@ class curve_pt(connector_pt):
         control2 = (self.box2.center[0] - control2[0] * bulge, self.box2.center[1] - control2[1] * bulge)
 
         connector_pt.__init__(self,
-               [path.normsubpath([path.normcurve(*(self.box1.center +
+               [path.normsubpath([path.normcurve_pt(*(self.box1.center +
                                                    control1 +
                                                    control2 + self.box2.center))], 0)])
 
@@ -339,12 +333,8 @@ class line(line_pt):
 
     """a line is the straight connector between the centers of two boxes"""
 
-    def __init__(self, box1, box2, boxdists=[0,0]):
-
-        boxdists_pt = (unit.topt(helper.getitemno(boxdists, 0)),
-                       unit.topt(helper.getitemno(boxdists, 1)))
-
-        line_pt.__init__(self, box1, box2, boxdists=boxdists_pt)
+    def __init__(self, box1, box2, boxdists=(0,0)):
+        line_pt.__init__(self, box1, box2, boxdists=map(unit.topt, boxdists))
 
 
 class curve(curve_pt):
@@ -358,15 +348,11 @@ class curve(curve_pt):
                  absangle1=None, absangle2=None,
                  absbulge=0, relbulge=0.39,
                  boxdists=[0,0]):
-
-        boxdists_pt = (unit.topt(helper.getitemno(boxdists, 0)),
-                       unit.topt(helper.getitemno(boxdists, 1)))
-
         curve_pt.__init__(self, box1, box2,
                           relangle1=relangle1, relangle2=relangle2,
                           absangle1=absangle1, absangle2=absangle2,
                           absbulge=unit.topt(absbulge), relbulge=relbulge,
-                          boxdists=boxdists_pt)
+                          boxdists=map(unit.topt, boxdists))
 
 class arc(arc_pt):
 
@@ -378,16 +364,12 @@ class arc(arc_pt):
 
     def __init__(self, box1, box2, relangle=45,
                  absbulge=None, relbulge=None, boxdists=[0,0]):
-
-        boxdists_pt = (unit.topt(helper.getitemno(boxdists, 0)),
-                       unit.topt(helper.getitemno(boxdists, 1)))
-
         if absbulge is not None:
             absbulge = unit.topt(absbulge)
         arc_pt.__init__(self, box1, box2,
                         relangle=relangle,
                         absbulge=absbulge, relbulge=relbulge,
-                        boxdists=boxdists_pt)
+                        boxdists=map(unit.topt, boxdists))
 
 
 class twolines(twolines_pt):
@@ -405,10 +387,6 @@ class twolines(twolines_pt):
                  bezierradius=None, beziersoftness=1,
                  arcradius=None,
                  boxdists=[0,0]):
-
-        boxdists_pt = (unit.topt(helper.getitemno(boxdists, 0)),
-                       unit.topt(helper.getitemno(boxdists, 1)))
-
         if length1 is not None:
             length1 = unit.topt(length1)
         if length2 is not None:
@@ -424,7 +402,7 @@ class twolines(twolines_pt):
                              length1=length1, length2=length2,
                              bezierradius=bezierradius, beziersoftness=1,
                              arcradius=arcradius,
-                             boxdists=boxdists_pt)
+                             boxdists=map(unit.topt, boxdists))
 
 
 
