@@ -1,8 +1,7 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
-# Copyright (C) 2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2004-2005 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -18,13 +17,16 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
-__all__ = ["data", "graph", "key", "style"]
-
-
-for module in __all__:
+__allmodules__ = ["data", "key", "style", "axis"]
+for module in __allmodules__:
     __import__(module, globals(), locals(), [])
 
-from graph import graphxy
+import graph
+__allgraph__ = ["graphxy"]
+for importfromgraph in __allgraph__:
+    locals()[importfromgraph] = getattr(graph, importfromgraph)
+
+__all__ = __allmodules__ + __allgraph__
