@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
 # Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
 # Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
-# Copyright (C) 2002-2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2005 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -20,7 +19,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
 import sys
@@ -166,6 +165,10 @@ class rational:
     def __abs__(self):
         return rational((abs(self.num), abs(self.denom)))
 
+    def __add__(self, other):
+        assert abs(other) < 1e-10
+        return float(self)
+
     def __mul__(self, other):
         return rational((self.num * other.num, self.denom * other.denom))
 
@@ -223,7 +226,7 @@ class tick(rational):
             self.label = other.label
 
 
-def mergeticklists(list1, list2, keepfirstifequal=0):
+def mergeticklists(list1, list2, mergeequal=1):
     """helper function to merge tick lists
     - return a merged list of ticks out of list1 and list2
     - CAUTION: original lists have to be ordered
@@ -241,7 +244,7 @@ def mergeticklists(list1, list2, keepfirstifequal=0):
                i += 1
                j += 1
             if list2[j] == list1[i]: # merge tick
-               if not keepfirstifequal:
+               if mergeequal:
                    list1[i].merge(list2[j])
                j += 1
             i += 1
