@@ -1,10 +1,9 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
 # Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
 # Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
-# Copyright (C) 2002-2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2006 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -20,124 +19,20 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
-import math, re, string
+import math, re, string, warnings
 from pyx import canvas, path, trafo, unit
 from pyx.graph import style
-from pyx.graph.axis import painter, axis
+from pyx.graph.axis import axis, positioner
 
 
 goldenmean = 0.5 * (math.sqrt(5) + 1)
 
 
-class lineaxispos:
-    """an axispos linear along a line with a fix direction for the ticks"""
-
-    def __init__(self, convert, x1, y1, x2, y2, fixtickdirection):
-        """initializes the instance
-        - only the convert method is needed from the axis
-        - x1, y1, x2, y2 are PyX lengths (start and end position of the line)
-        - fixtickdirection is a tuple tick direction (fixed along the line)"""
-        self.convert = convert
-        self.x1 = x1
-        self.y1 = y1
-        self.x2 = x2
-        self.y2 = y2
-        self.x1_pt = unit.topt(x1)
-        self.y1_pt = unit.topt(y1)
-        self.x2_pt = unit.topt(x2)
-        self.y2_pt = unit.topt(y2)
-        self.fixtickdirection = fixtickdirection
-
-    def vbasepath(self, v1=None, v2=None):
-        if v1 is None:
-            v1 = 0
-        if v2 is None:
-            v2 = 1
-        return path.line_pt((1-v1)*self.x1_pt+v1*self.x2_pt,
-                            (1-v1)*self.y1_pt+v1*self.y2_pt,
-                            (1-v2)*self.x1_pt+v2*self.x2_pt,
-                            (1-v2)*self.y1_pt+v2*self.y2_pt)
-
-    def basepath(self, x1=None, x2=None):
-        if x1 is None:
-            v1 = 0
-        else:
-            v1 = self.convert(x1)
-        if x2 is None:
-            v2 = 1
-        else:
-            v2 = self.convert(x2)
-        return path.line_pt((1-v1)*self.x1_pt+v1*self.x2_pt,
-                            (1-v1)*self.y1_pt+v1*self.y2_pt,
-                            (1-v2)*self.x1_pt+v2*self.x2_pt,
-                            (1-v2)*self.y1_pt+v2*self.y2_pt)
-
-    def gridpath(self, x):
-        raise RuntimeError("gridpath not available")
-
-    def vgridpath(self, v):
-        raise RuntimeError("gridpath not available")
-
-    def vtickpoint_pt(self, v):
-        return (1-v)*self.x1_pt+v*self.x2_pt, (1-v)*self.y1_pt+v*self.y2_pt
-
-    def vtickpoint(self, v):
-        return (1-v)*self.x1+v*self.x2, (1-v)*self.y1+v*self.y2
-
-    def tickpoint_pt(self, x):
-        v = self.convert(x)
-        return (1-v)*self.x1_pt+v*self.x2_pt, (1-v)*self.y1_pt+v*self.y2_pt
-
-    def tickpoint(self, x):
-        v = self.convert(x)
-        return (1-v)*self.x1+v*self.x2, (1-v)*self.y1+v*self.y2
-
-    def tickdirection(self, x):
-        return self.fixtickdirection
-
-    def vtickdirection(self, v):
-        return self.fixtickdirection
-
-
-class lineaxisposlinegrid(lineaxispos):
-    """an axispos linear along a line with a fix direction for the ticks
-    with support for grid lines for a rectangular graphs"""
-
-    __implements__ = painter._Iaxispos
-
-    def __init__(self, convert, x1, y1, x2, y2, fixtickdirection, startgridlength, endgridlength):
-        """initializes the instance
-        - only the convert method is needed from the axis
-        - x1, y1, x2, y2 are PyX lengths (start and end position of the line)
-        - fixtickdirection is a tuple tick direction (fixed along the line)
-        - startgridlength and endgridlength are PyX lengths for the starting
-          and end point of the grid, respectively; the gridpath is a line along
-          the fixtickdirection"""
-        lineaxispos.__init__(self, convert, x1, y1, x2, y2, fixtickdirection)
-        self.startgridlength = startgridlength
-        self.endgridlength = endgridlength
-        self.startgridlength_pt = unit.topt(self.startgridlength)
-        self.endgridlength_pt = unit.topt(self.endgridlength)
-
-    def gridpath(self, x):
-        v = self.convert(x)
-        return path.line_pt((1-v)*self.x1_pt+v*self.x2_pt+self.fixtickdirection[0]*self.startgridlength_pt,
-                          (1-v)*self.y1_pt+v*self.y2_pt+self.fixtickdirection[1]*self.startgridlength_pt,
-                          (1-v)*self.x1_pt+v*self.x2_pt+self.fixtickdirection[0]*self.endgridlength_pt,
-                          (1-v)*self.y1_pt+v*self.y2_pt+self.fixtickdirection[1]*self.endgridlength_pt)
-
-    def vgridpath(self, v):
-        return path.line_pt((1-v)*self.x1_pt+v*self.x2_pt+self.fixtickdirection[0]*self.startgridlength_pt,
-                          (1-v)*self.y1_pt+v*self.y2_pt+self.fixtickdirection[1]*self.startgridlength_pt,
-                          (1-v)*self.x1_pt+v*self.x2_pt+self.fixtickdirection[0]*self.endgridlength_pt,
-                          (1-v)*self.y1_pt+v*self.y2_pt+self.fixtickdirection[1]*self.endgridlength_pt)
-
-
 class styledata:
-    """Styledata storage class
+    """style data storage class
 
     Instances of this class are used to store data from the styles
     and to pass point data to the styles by instances named privatedata
@@ -152,6 +47,7 @@ class plotitem:
 
     def __init__(self, graph, data, styles):
         self.data = data
+        self.title = data.title
 
         # add styles to ensure all needs of the given styles
         provided = [] # already provided sharedata variables
@@ -169,27 +65,46 @@ class plotitem:
         self.privatedatalist = [styledata() for s in self.styles]
 
         # perform setcolumns to all styles
-        self.data.initplotitem(self, graph)
-        columns = self.data.getcolumnnames(self)
-        usedcolumns = []
+        self.usedcolumnnames = []
         for privatedata, s in zip(self.privatedatalist, self.styles):
-            usedcolumns.extend(s.columns(privatedata, self.sharedata, graph, columns))
-        for column in columns:
-            if column not in usedcolumns:
-                raise ValueError("unused column '%s'" % column)
+            self.usedcolumnnames.extend(s.columnnames(privatedata, self.sharedata, graph, self.data.columnnames))
 
     def selectstyles(self, graph, selectindex, selecttotal):
         for privatedata, style in zip(self.privatedatalist, self.styles):
             style.selectstyle(privatedata, self.sharedata, graph, selectindex, selecttotal)
 
-    def adjustaxes(self, graph, step):
-        self.data.adjustaxes(self, graph, step)
+    def adjustaxesstatic(self, graph):
+        for columnname, data in self.data.columns.items():
+            for privatedata, style in zip(self.privatedatalist, self.styles):
+                style.adjustaxis(privatedata, self.sharedata, graph, columnname, data)
+
+    def makedynamicdata(self, graph):
+        self.dynamiccolumns = self.data.dynamiccolumns(graph)
+
+    def adjustaxesdynamic(self, graph):
+        for columnname, data in self.dynamiccolumns.items():
+            for privatedata, style in zip(self.privatedatalist, self.styles):
+                style.adjustaxis(privatedata, self.sharedata, graph, columnname, data)
 
     def draw(self, graph):
-        self.data.draw(self, graph)
-
-    def gettitle(self):
-        return self.data.gettitle()
+        for privatedata, style in zip(self.privatedatalist, self.styles):
+            style.initdrawpoints(privatedata, self.sharedata, graph)
+        point = {}
+        useitems = []
+        for columnname in self.usedcolumnnames:
+            try:
+                useitems.append((columnname, self.dynamiccolumns[columnname]))
+            except KeyError:
+                useitems.append((columnname, self.data.columns[columnname]))
+        if not useitems:
+            raise ValueError("cannot draw empty data")
+        for i in xrange(len(useitems[0][1])):
+            for columnname, data in useitems:
+                point[columnname] = data[i]
+            for privatedata, style in zip(self.privatedatalist, self.styles):
+                style.drawpoint(privatedata, self.sharedata, graph, point)
+        for privatedata, style in zip(self.privatedatalist, self.styles):
+            style.donedrawpoints(privatedata, self.sharedata, graph)
 
     def key_pt(self, graph, x_pt, y_pt, width_pt, height_pt):
         for privatedata, style in zip(self.privatedatalist, self.styles):
@@ -204,15 +119,53 @@ class plotitem:
             return stylesdata
         elif len(stylesdata) == 1:
             return stylesdata[0]
-        raise AttributeError("access to styledata attributes failed")
+        raise AttributeError("access to styledata attribute '%s' failed" % attr)
 
 
+class graph(canvas.canvas):
 
-class graphxy(canvas.canvas):
+    def __init__(self):
+        canvas.canvas.__init__(self)
+        self.axes = {}
+        self.plotitems = []
+        self._calls = {}
+        self.didranges = 0
+        self.diddata = 0
 
-    def plot(self, data, styles=None):
-        if self.haslayout:
-            raise RuntimeError("layout setup was already performed")
+    def did(self, method, *args, **kwargs):
+        if not self._calls.has_key(method):
+            self._calls[method] = []
+        for callargs in self._calls[method]:
+            if callargs == (args, kwargs):
+                return 1
+        self._calls[method].append((args, kwargs))
+        return 0
+
+    def bbox(self):
+        self.finish()
+        return canvas.canvas.bbox(self)
+
+    def registerPS(self, registry):
+        self.finish()
+        canvas.canvas.registerPS(self, registry)
+
+    def registerPDF(self, registry):
+        self.finish()
+        canvas.canvas.registerPDF(self, registry)
+
+    def processPS(self, file, writer, context, registry, bbox):
+        self.finish()
+        canvas.canvas.processPS(self, file, writer, context, registry, bbox)
+
+    def processPDF(self, file, writer, context, registry, bbox):
+        self.finish()
+        canvas.canvas.processPDF(self, file, writer, context, registry, bbox)
+
+    def plot(self, data, styles=None, rangewarning=1):
+        if self.didranges and rangewarning:
+            warnings.warn("axes ranges have already been analysed; no further adjustments will be performed")
+        if self.diddata:
+            raise RuntimeError("can't plot further data after dodata() has been executed")
         singledata = 0
         try:
             for d in data:
@@ -225,37 +178,183 @@ class graphxy(canvas.canvas):
         if styles is None:
             for d in usedata:
                 if styles is None:
-                    styles = d.getdefaultstyles()
-                elif styles != d.getdefaultstyles():
+                    styles = d.defaultstyles
+                elif styles != d.defaultstyles:
                     raise RuntimeError("defaultstyles differ")
         plotitems = []
         for d in usedata:
             plotitems.append(plotitem(self, d, styles))
         self.plotitems.extend(plotitems)
+        if self.didranges:
+            for aplotitem in plotitems:
+                aplotitem.makedynamicdata(self)
         if singledata:
             return plotitems[0]
         else:
             return plotitems
 
+    def doranges(self):
+        if self.did(self.doranges):
+            return
+        for plotitem in self.plotitems:
+            plotitem.adjustaxesstatic(self)
+        for plotitem in self.plotitems:
+            plotitem.makedynamicdata(self)
+        for plotitem in self.plotitems:
+            plotitem.adjustaxesdynamic(self)
+        self.didranges = 1
+
+    def dolayout(self):
+        raise NotImplementedError
+
+    def dobackground(self):
+        raise NotImplementedError
+
+    def doaxes(self):
+        raise NotImplementedError
+
+    def dodata(self):
+        if self.did(self.dodata):
+            return
+        self.dolayout()
+        self.dobackground()
+
+        # count the usage of styles and perform selects
+        styletotal = {}
+        def stylesid(styles):
+            return ":".join([str(id(style)) for style in styles])
+        for plotitem in self.plotitems:
+            try:
+                styletotal[stylesid(plotitem.styles)] += 1
+            except:
+                styletotal[stylesid(plotitem.styles)] = 1
+        styleindex = {}
+        for plotitem in self.plotitems:
+            try:
+                styleindex[stylesid(plotitem.styles)] += 1
+            except:
+                styleindex[stylesid(plotitem.styles)] = 0
+            plotitem.selectstyles(self, styleindex[stylesid(plotitem.styles)],
+                                        styletotal[stylesid(plotitem.styles)])
+
+        for plotitem in self.plotitems:
+            plotitem.draw(self)
+
+        self.diddata = 1
+
+    def dokey(self):
+        raise NotImplementedError
+
+    def finish(self):
+        self.dobackground()
+        self.doaxes()
+        self.dodata()
+        self.dokey()
+
+
+class graphxy(graph):
+
+    def __init__(self, xpos=0, ypos=0, width=None, height=None, ratio=goldenmean,
+                 key=None, backgroundattrs=None, axesdist=0.8*unit.v_cm,
+                 xaxisat=None, yaxisat=None, **axes):
+        graph.__init__(self)
+
+        self.xpos = xpos
+        self.ypos = ypos
+        self.xpos_pt = unit.topt(self.xpos)
+        self.ypos_pt = unit.topt(self.ypos)
+        self.xaxisat = xaxisat
+        self.yaxisat = yaxisat
+        self.key = key
+        self.backgroundattrs = backgroundattrs
+        self.axesdist_pt = unit.topt(axesdist)
+
+        self.width = width
+        self.height = height
+        if width is None:
+            if height is None:
+                raise ValueError("specify width and/or height")
+            else:
+                self.width = ratio * self.height
+        elif height is None:
+            self.height = (1.0/ratio) * self.width
+        self.width_pt = unit.topt(self.width)
+        self.height_pt = unit.topt(self.height)
+
+        for axisname, aaxis in axes.items():
+            if aaxis is not None:
+                if not isinstance(aaxis, axis.linkedaxis):
+                    self.axes[axisname] = axis.anchoredaxis(aaxis, self.texrunner, axisname)
+                else:
+                    self.axes[axisname] = aaxis
+        for axisname, axisat in [("x", xaxisat), ("y", yaxisat)]:
+            okey = axisname + "2"
+            if not axes.has_key(axisname):
+                if not axes.has_key(okey):
+                    self.axes[axisname] = axis.anchoredaxis(axis.linear(), self.texrunner, axisname)
+                    self.axes[okey] = axis.linkedaxis(self.axes[axisname], okey)
+                else:
+                    self.axes[axisname] = axis.linkedaxis(self.axes[okey], axisname)
+            elif not axes.has_key(okey) and axisat is None:
+                self.axes[okey] = axis.linkedaxis(self.axes[axisname], okey)
+
+        if self.axes.has_key("x"):
+            self.xbasepath = self.axes["x"].basepath
+            self.xvbasepath = self.axes["x"].vbasepath
+            self.xgridpath = self.axes["x"].gridpath
+            self.xtickpoint_pt = self.axes["x"].tickpoint_pt
+            self.xtickpoint = self.axes["x"].tickpoint
+            self.xvtickpoint_pt = self.axes["x"].vtickpoint_pt
+            self.xvtickpoint = self.axes["x"].tickpoint
+            self.xtickdirection = self.axes["x"].tickdirection
+            self.xvtickdirection = self.axes["x"].vtickdirection
+
+        if self.axes.has_key("y"):
+            self.ybasepath = self.axes["y"].basepath
+            self.yvbasepath = self.axes["y"].vbasepath
+            self.ygridpath = self.axes["y"].gridpath
+            self.ytickpoint_pt = self.axes["y"].tickpoint_pt
+            self.ytickpoint = self.axes["y"].tickpoint
+            self.yvtickpoint_pt = self.axes["y"].vtickpoint_pt
+            self.yvtickpoint = self.axes["y"].tickpoint
+            self.ytickdirection = self.axes["y"].tickdirection
+            self.yvtickdirection = self.axes["y"].vtickdirection
+
+        self.axesnames = ([], [])
+        for axisname, aaxis in self.axes.items():
+            if axisname[0] not in "xy" or (len(axisname) != 1 and (not axisname[1:].isdigit() or
+                                                                   axisname[1:] == "1")):
+                raise ValueError("invalid axis name")
+            if axisname[0] == "x":
+                self.axesnames[0].append(axisname)
+            else:
+                self.axesnames[1].append(axisname)
+            aaxis.setcreatecall(self.doaxiscreate, axisname)
+
+
     def pos_pt(self, x, y, xaxis=None, yaxis=None):
         if xaxis is None:
             xaxis = self.axes["x"]
         if yaxis is None:
             yaxis = self.axes["y"]
-        return self.xpos_pt + xaxis.convert(x)*self.width_pt, self.ypos_pt + yaxis.convert(y)*self.height_pt
+        return (self.xpos_pt + xaxis.convert(x)*self.width_pt,
+                self.ypos_pt + yaxis.convert(y)*self.height_pt)
 
     def pos(self, x, y, xaxis=None, yaxis=None):
         if xaxis is None:
             xaxis = self.axes["x"]
         if yaxis is None:
             yaxis = self.axes["y"]
-        return self.xpos + xaxis.convert(x)*self.width, self.ypos + yaxis.convert(y)*self.height
+        return (self.xpos + xaxis.convert(x)*self.width,
+                self.ypos + yaxis.convert(y)*self.height)
 
     def vpos_pt(self, vx, vy):
-        return self.xpos_pt + vx*self.width_pt, self.ypos_pt + vy*self.height_pt
+        return (self.xpos_pt + vx*self.width_pt,
+                self.ypos_pt + vy*self.height_pt)
 
     def vpos(self, vx, vy):
-        return self.xpos + vx*self.width, self.ypos + vy*self.height
+        return (self.xpos + vx*self.width,
+                self.ypos + vy*self.height)
 
     def vgeodesic(self, vx1, vy1, vx2, vy2):
         """returns a geodesic path between two points in graph coordinates"""
@@ -285,137 +384,101 @@ class graphxy(canvas.canvas):
         else:
             raise ValueError("direction invalid")
 
-    def keynum(self, key):
-        try:
-            while key[0] in string.letters:
-                key = key[1:]
-            return int(key)
-        except IndexError:
-            return 1
+    def xvgridpath(self, vx):
+        return path.line_pt(self.xpos_pt + vx*self.width_pt, self.ypos_pt,
+                            self.xpos_pt + vx*self.width_pt, self.ypos_pt + self.height_pt)
 
-    def removedomethod(self, method):
-        hadmethod = 0
-        while 1:
-            try:
-                self.domethods.remove(method)
-                hadmethod = 1
-            except ValueError:
-                return hadmethod
+    def yvgridpath(self, vy):
+        return path.line_pt(self.xpos_pt, self.ypos_pt + vy*self.height_pt,
+                            self.xpos_pt + self.width_pt, self.ypos_pt + vy*self.height_pt)
+
+    def axistrafo(self, axis, t):
+        c = canvas.canvas([t])
+        c.insert(axis.canvas)
+        axis.canvas = c
+
+    def axisatv(self, axis, v):
+        if axis.positioner.fixtickdirection[0]:
+            # it is a y-axis
+            self.axistrafo(axis, trafo.translate_pt(self.xpos_pt + v*self.width_pt - axis.positioner.x1_pt, 0))
+        else:
+            # it is an x-axis
+            self.axistrafo(axis, trafo.translate_pt(0, self.ypos_pt + v*self.height_pt - axis.positioner.y1_pt))
+
+    def doaxispositioner(self, axisname):
+        if self.did(self.doaxispositioner, axisname):
+            return
+        self.doranges()
+        if axisname == "x":
+            self.axes["x"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt,
+                                                                   self.xpos_pt + self.width_pt, self.ypos_pt,
+                                                                   (0, 1), self.xvgridpath))
+        elif axisname == "x2":
+            self.axes["x2"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt + self.height_pt,
+                                                                    self.xpos_pt + self.width_pt, self.ypos_pt + self.height_pt,
+                                                                    (0, -1), self.xvgridpath))
+        elif axisname == "y":
+            self.axes["y"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt,
+                                                                   self.xpos_pt, self.ypos_pt + self.height_pt,
+                                                                   (1, 0), self.yvgridpath))
+        elif axisname == "y2":
+            self.axes["y2"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt + self.width_pt, self.ypos_pt,
+                                                                    self.xpos_pt + self.width_pt, self.ypos_pt + self.height_pt,
+                                                                    (-1, 0), self.yvgridpath))
+        else:
+            if axisname[1:] == "3":
+                dependsonaxisname = axisname[0]
+            else:
+                dependsonaxisname = "%s%d" % (axisname[0], int(axisname[1:]) - 2)
+            self.doaxiscreate(dependsonaxisname)
+            sign = 2*(int(axisname[1:]) % 2) - 1
+            if axisname[0] == "x":
+                y_pt = self.axes[dependsonaxisname].positioner.y1_pt - sign * (self.axes[dependsonaxisname].canvas.extent_pt + self.axesdist_pt)
+                self.axes[axisname].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, y_pt,
+                                                                            self.xpos_pt + self.width_pt, y_pt,
+                                                                            (0, sign), self.xvgridpath))
+            else:
+                x_pt = self.axes[dependsonaxisname].positioner.x1_pt - sign * (self.axes[dependsonaxisname].canvas.extent_pt + self.axesdist_pt)
+                self.axes[axisname].setpositioner(positioner.lineaxispos_pt(x_pt, self.ypos_pt,
+                                                                            x_pt, self.ypos_pt + self.height_pt,
+                                                                            (sign, 0), self.yvgridpath))
+
+    def doaxiscreate(self, axisname):
+        if self.did(self.doaxiscreate, axisname):
+            return
+        self.doaxispositioner(axisname)
+        self.axes[axisname].create()
 
     def dolayout(self):
-        if not self.removedomethod(self.dolayout): return
-
-        # count the usage of styles and perform selects
-        styletotal = {}
-        def stylesid(styles):
-            return ":".join([str(id(style)) for style in styles])
-        for plotitem in self.plotitems:
-            try:
-                styletotal[stylesid(plotitem.styles)] += 1
-            except:
-                styletotal[stylesid(plotitem.styles)] = 1
-        styleindex = {}
-        for plotitem in self.plotitems:
-            try:
-                styleindex[stylesid(plotitem.styles)] += 1
-            except:
-                styleindex[stylesid(plotitem.styles)] = 0
-            plotitem.selectstyles(self, styleindex[stylesid(plotitem.styles)], styletotal[stylesid(plotitem.styles)])
-
-        # adjust the axes ranges
-        for step in range(3):
-            for plotitem in self.plotitems:
-                plotitem.adjustaxes(self, step)
-
-        # finish all axes
-        XPattern = re.compile(r"x([2-9]|[1-9][0-9]+)?$")
-        YPattern = re.compile(r"y([2-9]|[1-9][0-9]+)?$")
-        xaxisextents = [0, 0]
-        yaxisextents = [0, 0]
-        needxaxisdist = [0, 0]
-        needyaxisdist = [0, 0]
-        items = list(self.axes.items())
-        items.sort() #TODO: alphabetical sorting breaks for axis numbers bigger than 9
-        # TODO: linked axes are not taken into account (consider x being a link to x2)
-        for key, axis in items:
-            num = self.keynum(key)
-            num2 = 1 - num % 2 # x1 -> 0, x2 -> 1, x3 -> 0, x4 -> 1, ...
-            num3 = 2 * (num % 2) - 1 # x1 -> 1, x2 -> -1, x3 -> 1, x4 -> -1, ...
-            if XPattern.match(key):
-                if needxaxisdist[num2]:
-                    xaxisextents[num2] += self.axesdist
-                self.axespos[key] = lineaxisposlinegrid(axis.convert,
-                                                        self.xpos,
-                                                        self.ypos + num2*self.height - num3*xaxisextents[num2],
-                                                        self.xpos + self.width,
-                                                        self.ypos + num2*self.height - num3*xaxisextents[num2],
-                                                        (0, num3),
-                                                        xaxisextents[num2], xaxisextents[num2] + self.height)
-                if num == 1:
-                    self.xbasepath = self.axespos[key].basepath
-                    self.xvbasepath = self.axespos[key].vbasepath
-                    self.xgridpath = self.axespos[key].gridpath
-                    self.xvgridpath = self.axespos[key].vgridpath
-                    self.xtickpoint_pt = self.axespos[key].tickpoint_pt
-                    self.xtickpoint = self.axespos[key].tickpoint
-                    self.xvtickpoint_pt = self.axespos[key].vtickpoint_pt
-                    self.xvtickpoint = self.axespos[key].tickpoint
-                    self.xtickdirection = self.axespos[key].tickdirection
-                    self.xvtickdirection = self.axespos[key].vtickdirection
-            elif YPattern.match(key):
-                if needyaxisdist[num2]:
-                    yaxisextents[num2] += self.axesdist
-                self.axespos[key] = lineaxisposlinegrid(axis.convert,
-                                                        self.xpos + num2*self.width - num3*yaxisextents[num2],
-                                                        self.ypos,
-                                                        self.xpos + num2*self.width - num3*yaxisextents[num2],
-                                                        self.ypos + self.height,
-                                                        (num3, 0),
-                                                        yaxisextents[num2], yaxisextents[num2] + self.width)
-                if num == 1:
-                    self.ybasepath = self.axespos[key].basepath
-                    self.yvbasepath = self.axespos[key].vbasepath
-                    self.ygridpath = self.axespos[key].gridpath
-                    self.yvgridpath = self.axespos[key].vgridpath
-                    self.ytickpoint_pt = self.axespos[key].tickpoint_pt
-                    self.ytickpoint = self.axespos[key].tickpoint
-                    self.yvtickpoint_pt = self.axespos[key].vtickpoint_pt
-                    self.yvtickpoint = self.axespos[key].tickpoint
-                    self.ytickdirection = self.axespos[key].tickdirection
-                    self.yvtickdirection = self.axespos[key].vtickdirection
-            else:
-                raise ValueError("Axis key '%s' not allowed" % key)
-            axis.finish(self.axespos[key])
-            if XPattern.match(key):
-                xaxisextents[num2] += axis.axiscanvas.extent
-                needxaxisdist[num2] = 1
-            if YPattern.match(key):
-                yaxisextents[num2] += axis.axiscanvas.extent
-                needyaxisdist[num2] = 1
-        self.haslayout = 1
+        if self.did(self.dolayout):
+            return
+        for axisname in self.axes.keys():
+            self.doaxiscreate(axisname)
+        if self.xaxisat is not None:
+            self.axisatv(self.axes["x"], self.axes["y"].convert(self.xaxisat))
+        if self.yaxisat is not None:
+            self.axisatv(self.axes["y"], self.axes["x"].convert(self.yaxisat))
 
     def dobackground(self):
-        self.dolayout()
-        if not self.removedomethod(self.dobackground): return
+        if self.did(self.dobackground):
+            return
         if self.backgroundattrs is not None:
             self.draw(path.rect_pt(self.xpos_pt, self.ypos_pt, self.width_pt, self.height_pt),
                       self.backgroundattrs)
 
     def doaxes(self):
+        if self.did(self.doaxes):
+            return
         self.dolayout()
-        if not self.removedomethod(self.doaxes): return
+        self.dobackground()
         for axis in self.axes.values():
-            self.insert(axis.axiscanvas)
-
-    def dodata(self):
-        self.dolayout()
-        if not self.removedomethod(self.dodata): return
-        for plotitem in self.plotitems:
-            plotitem.draw(self)
+            self.insert(axis.canvas)
 
     def dokey(self):
-        self.dolayout()
-        if not self.removedomethod(self.dokey): return
+        if self.did(self.dokey):
+            return
+        self.dobackground()
+        self.dodata()
         if self.key is not None:
             c = self.key.paint(self.plotitems)
             bbox = c.bbox()
@@ -423,87 +486,14 @@ class graphxy(canvas.canvas):
                 ppos = pmin+0.5*(cmax-cmin)+dist+pos*(pmax-pmin-cmax+cmin-2*dist)
                 cpos = 0.5*(cmin+cmax)+(1-inside)*(1-2*pos)*(cmax-cmin+2*dist)
                 return ppos-cpos
-            x = parentchildalign(self.xpos_pt, self.xpos_pt+self.width_pt,
-                                 bbox.llx_pt, bbox.urx_pt,
-                                 self.key.hpos, unit.topt(self.key.hdist), self.key.hinside)
-            y = parentchildalign(self.ypos_pt, self.ypos_pt+self.height_pt,
-                                 bbox.lly_pt, bbox.ury_pt,
-                                 self.key.vpos, unit.topt(self.key.vdist), self.key.vinside)
-            self.insert(c, [trafo.translate_pt(x, y)])
-
-    def finish(self):
-        while len(self.domethods):
-            self.domethods[0]()
-
-    def initwidthheight(self, width, height, ratio):
-        if (width is not None) and (height is None):
-             self.width = width
-             self.height = (1.0/ratio) * self.width
-        elif (height is not None) and (width is None):
-             self.height = height
-             self.width = ratio * self.height
-        else:
-             self.width = width
-             self.height = height
-        self.width_pt = unit.topt(self.width)
-        self.height_pt = unit.topt(self.height)
-        if self.width_pt <= 0: raise ValueError("width <= 0")
-        if self.height_pt <= 0: raise ValueError("height <= 0")
-
-    def initaxes(self, axes, addlinkaxes=0):
-        for key in ["x", "y"]:
-            if not axes.has_key(key):
-                axes[key] = axis.linear()
-            elif axes[key] is None:
-                del axes[key]
-            if addlinkaxes:
-                if not axes.has_key(key + "2") and axes.has_key(key):
-                    axes[key + "2"] = axes[key].createlinkaxis()
-                elif axes[key + "2"] is None:
-                    del axes[key + "2"]
-        self.axes = axes
-        self.axesnames = ([], [])
-        for key in axes.keys():
-            if len(key) != 1 and (not key[1:].isdigit() or key[1:] == "1"):
-                raise ValueError("invalid axis count")
-            if key[0] == "x":
-                self.axesnames[0].append(key)
-            elif key[0] == "y":
-                self.axesnames[1].append(key)
-            else:
-                raise ValueError("invalid axis name")
-
-    def __init__(self, xpos=0, ypos=0, width=None, height=None, ratio=goldenmean,
-                 key=None, backgroundattrs=None, axesdist=0.8*unit.v_cm, **axes):
-        canvas.canvas.__init__(self)
-        self.xpos = xpos
-        self.ypos = ypos
-        self.xpos_pt = unit.topt(self.xpos)
-        self.ypos_pt = unit.topt(self.ypos)
-        self.initwidthheight(width, height, ratio)
-        self.initaxes(axes, 1)
-        self.axescanvas = {}
-        self.axespos = {}
-        self.key = key
-        self.backgroundattrs = backgroundattrs
-        self.axesdist = axesdist
-        self.plotitems = []
-        self.domethods = [self.dolayout, self.dobackground, self.doaxes, self.dodata, self.dokey]
-        self.haslayout = 0
-        self.addkeys = []
-
-    def bbox(self):
-        self.finish()
-        return canvas.canvas.bbox(self)
-
-    def prolog(self):
-        self.finish()
-        return canvas.canvas.prolog(self)
-
-    def outputPS(self, file):
-        self.finish()
-        canvas.canvas.outputPS(self, file)
-
+            if bbox:
+                x = parentchildalign(self.xpos_pt, self.xpos_pt+self.width_pt,
+                                     bbox.llx_pt, bbox.urx_pt,
+                                     self.key.hpos, unit.topt(self.key.hdist), self.key.hinside)
+                y = parentchildalign(self.ypos_pt, self.ypos_pt+self.height_pt,
+                                     bbox.lly_pt, bbox.ury_pt,
+                                     self.key.vpos, unit.topt(self.key.vdist), self.key.vinside)
+                self.insert(c, [trafo.translate_pt(x, y)])
 
 
 # some thoughts, but deferred right now
