@@ -1,8 +1,7 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
-# Copyright (C) 2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2004-2005 Andr Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -18,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 # This file configures PyX search paths relative to the current
@@ -32,5 +31,5 @@ import os
 
 lfsdir = os.path.join(os.path.dirname(__file__), "lfs")
 sharedir = os.path.join(os.path.dirname(__file__), "..", "contrib")
-pyxrc = os.path.join(os.path.dirname(__file__), "..", "pyxrc")
+pyxrcdir = os.path.join(os.path.dirname(__file__), "..")
 
