@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: ISO-8859-1 -*-
 #
 #
@@ -19,10 +18,9 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 import types
-import helper
 
 scale = { 't':1, 'u':1, 'v':1, 'w':1, 'x':1 }
 
@@ -84,8 +82,8 @@ class length:
     w=width, and x=TeX) which can be scaled separately (except for the true
     component, which is always unscaled). Lengths can be constructed in units
     of "pt", "mm", "cm", "m" and "inch". When no unit is given, a module
-    default is used, which can be changed with the help of the set method of
-    the present module.
+    default is used, which can be changed with the help of the module level function
+    set().
     """
 
     def __init__(self, f=0, type="u", unit=None):
@@ -93,7 +91,7 @@ class length:
         in the given unit. If unit is not set, the currently set default unit is used.
         """
         self.t = self.u = self.v = self.w = self.x = 0
-        l = f * _m[unit or _default_unit]
+        l = float(f) * _m[unit or _default_unit]
         if type == "t":
             self.t = l
         elif type == "u":
@@ -106,7 +104,13 @@ class length:
             self.x = l
 
     def __cmp__(self, other):
-        return cmp(tom(self), tom(other))
+        # we try to convert self and other into meters and
+        # if this fails, we give other a chance to do the comparison
+        try:
+            return cmp(tom(self), tom(other))
+        except:
+            # why does -cmp(other, self) not work?
+            return -other.__cmp__(self)
 
     def __mul__(self, factor):
         result = length()
@@ -119,21 +123,26 @@ class length:
 
     __rmul__=__mul__
 
-    def __div__(self, factor):
-        if isinstance(factor, length):
-            return tom(self) / tom(factor)
+    def __div__(self, divisor):
+        if isinstance(divisor, length):
+            return tom(self) / tom(divisor)
         result = length()
-        result.t = self.t / factor
-        result.u = self.u / factor
-        result.v = self.v / factor
-        result.w = self.w / factor
-        result.x = self.x / factor
+        result.t = self.t / divisor
+        result.u = self.u / divisor
+        result.v = self.v / divisor
+        result.w = self.w / divisor
+        result.x = self.x / divisor
         return result
 
     def __add__(self, other):
         # convert to length if necessary
         if not isinstance(other, length):
-            other = length(other)
+            # if other is not a length, we try to convert it into a length and
+            # if this fails, we give other a chance to do the addition
+            try:
+                other = length(other)
+            except:
+                return other + self
         result = length()
         result.t = self.t + other.t
         result.u = self.u + other.u
@@ -142,12 +151,17 @@ class length:
         result.x = self.x + other.x
         return result
 
-    __radd__=__add__
+    __radd__ = __add__
 
     def __sub__(self, other):
         # convert to length if necessary
         if not isinstance(other, length):
-            other = length(other)
+            # if other is not a length, we try to convert it into a length and
+            # if this fails, we give other a chance to do the subtraction
+            try:
+                other = length(other)
+            except:
+                return -other + self
         result = length()
         result.t = self.t - other.t
         result.u = self.u - other.u
